#!/System/Index/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === Autostart
#
# This class will autogenerate some files.
#
# It can be used i.e. as replacement for xfce_autostart.
# =========================================================================== #
# require 'autostart'
# =========================================================================== #
require 'fileutils'
require 'opn'
begin
  require 'colours'
rescue LoadError; end
require 'convert_global_env'
require 'autostart/constants.rb'
require 'autostart/version/version.rb'

begin
  require 'roebe/classes/create_desktop_file.rb'
rescue LoadError; end # bl $ROEBE/classes/create_desktop_file.rb

class Autostart # Autostart.new

  include Colours

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      what = 'default'
    )
    reset
    autostart_what(what) if what
  end

  # ========================================================================= #
  # === reset
  # ========================================================================= #
  def reset
    @array_these_symlinks_were_not_found = []
    @be_verbose = true
  end

  # ========================================================================= #
  # === consider_reporting_missing_entries_to_the_user
  # ========================================================================= #
  def consider_reporting_missing_entries_to_the_user
    unless @array_these_symlinks_were_not_found.empty?
      e; opn; e 'We did find some missing entries. Perhaps you '\
                'may wish to correct these:'
      e
      @array_these_symlinks_were_not_found.each {|entry|
        e simp("  #{entry}")
      }; e
    end
  end

  # ========================================================================= #
  # === mkdir
  # ========================================================================= #
  def mkdir(i)
    i = File.expand_path(i) if i.include? '~' # If includes ~, use File.expand_path
    FileUtils.mkdir_p(i) unless File.exist?(i)
  end; alias ensure_that_this_directory_exists mkdir # === ensure_that_this_directory_exists

  # ========================================================================= #
  # === remove_file
  # ========================================================================= #
  def remove_file(i)
    opn; e red+'Removing file '+sfile(i)+red+' now.' if File.exist?(i)
    File.delete(i) if File.file?(i)
  end

  # ========================================================================= #
  # === red
  # ========================================================================= #
  def red
    Colours::RED
  end

  # ========================================================================= #
  # === copy
  # ========================================================================= #
  def copy(from, to)
    from = ConvertGlobalEnv[from] if from.include? '$'
    to   = ConvertGlobalEnv[to]   if to.include? '$'
    from = File.expand_path(from)
    to   = File.expand_path(to)
    FileUtils.copy(from, to)
  end

  # ========================================================================= #
  # === symlink
  #
  # This method accepts only one argument and will split it based on ' '.
  #  File.symlink(real_existing_location, new_location)
  # ========================================================================= #
  def symlink(i) # Expects string input.
    real_existing_location, new_location = i.split ' '
    if real_existing_location.include? '$'
      # We solve this very simply without regex.
      last_position = real_existing_location.index('/')
      real_value = ENV[real_existing_location[1, last_position-1]].dup
      real_existing_location = real_value+real_existing_location[last_position..-1]
    end
    if new_location.include? '$'
      new_location = ConvertGlobalEnv.convert new_location
    end
    new_location = File.expand_path(new_location) if new_location.include? '~'
    filename = File.basename(real_existing_location)
    new_location << '/'+filename
    if File.symlink?(new_location)
      readlink = File.readlink(new_location)
      remove_file(readlink) unless File.exist?(readlink)
    end 
    remove_file(new_location) if File.exist? new_location
    if File.exist?(real_existing_location)
      opn; e 'Trying to symlink '+sfile(real_existing_location)+
             ' to '+sfile(new_location)+' now.'
      unless File.exist? new_location
        File.symlink(real_existing_location, new_location)
      else
        opn; e 'Can not symlink because '+sfile(new_location)+' does not exist.'
      end
    else
      if @be_verbose
        opn; e 'Trying to symlink '+sfile(real_existing_location)+
               ' to '+sfile(new_location)
        opn; e 'but the file '+sfile(real_existing_location)+' does not '\
               'exist, thus we skip this action.'
        @array_these_symlinks_were_not_found << real_existing_location
      end
    end
  end

  # ========================================================================= #
  # === show_header
  # ========================================================================= #
  def show_header(
      for_this_program = :xfce
    )
    case for_this_program
    when :xfce
      for_this_program = 'XFCE'
    when :kde
      for_this_program = 'KDE'
    end
    output_header
    opn; e "Autostart for #{orange(for_this_program)}."
    output_header
  end

  # ========================================================================= #
  # === output_header                                 (cliner tag, liner tag)
  # ========================================================================= #
  def output_header
    e '=' * 80
  end

  # ========================================================================= #
  # === show_help
  # ========================================================================= #
  def show_help
    e 'This class (class Autostart) allows you to "autostart" some '\
      'applications in different DE/WMs.'
    e
    e 'Available autostart actions are:'
    e
    e '  - xfce'
    e '  - kde'
    e '  - mate'
    e
  end

  # ========================================================================= #
  # === autostart_kde                                               (kde tag)
  #
  # To invoke this method, do:
  #   autostart --kde
  # ========================================================================= #
  def autostart_kde
    show_header :kde
    # mkdir HSS['KDE_AUTOSTART']
    ensure_that_this_directory_exists KDE_AUTOSTART
    mkdir '~/.kde/share/config/'
    copy '$LINUX/KDE/konsolerc','~/.kde/share/config/'
    ARRAY_SYMLINK_THESE_ENTRIES_FOR_KDE.each {|entry|
      symlink(entry+' $KDE_AUTOSTART')
    }
  end

  # ========================================================================= #
  # === autostart_mate
  # ========================================================================= #
  def autostart_mate(
      use_these_programs = YAML.load_file(
        '/Users/x/DATA/PROGRAMMING_LANGUAGES/RUBY/src/roebe/lib/roebe/yaml/'\
        'autostart_these_programs.yml'
      )
    )
    show_header :mate
    home_dir = HOME_DIR+'.config/autostart'
    mkdir home_dir
    use_these_programs.each {|this_file|
      Roebe::CreateDesktopFile.new(this_file)
    }
  end

  # ========================================================================= #
  # === autostart_xfce                                              (xfce tag)
  #
  # This will usually default to programs such as:
  #   xchat, bluefish, konsole and mrxvt.
  # ========================================================================= #
  def autostart_xfce(
      i = ARRAY_SYMLINK_THESE_ENTRIES_FOR_XFCE
    )
    show_header :xfce
    # home_dir = '~/.config/autostart'
    home_dir = HOME_DIR+'.config/autostart'
    mkdir home_dir
    i.each {|entry|
      symlink(entry+' '+home_dir)
    }
  end

  # ========================================================================= #
  # === autostart_what
  # ========================================================================= #
  def autostart_what(i = DEFAULT_INPUT)
    i = DEFAULT_INPUT if i.nil?
    i = i.first if i.is_a? Array # Fetch first entry if it is an Array.
    case i.to_s # case tag
    # ======================================================================= #
    # === autostart help
    # ======================================================================= #
    when /^-?-?help/
      show_help
      exit
    # ======================================================================= #
    # === autostart mate
    # ======================================================================= #
    when /^-?-?mate$/
      autostart_mate
    # ======================================================================= #
    # === autostart xfce
    # ======================================================================= #
    when /^-?-?xfce$/,'default','' # This is the default since August 2013.
      autostart_xfce
    # ======================================================================= #
    # === autostart kde
    # ======================================================================= #
    when /^-?-?kde$/
      autostart_kde
    else
      opn; e 'Not found `'+simp(i)+'` in the list of available entries.'
      opn; e 'It is probably not registered.'
    end
    consider_reporting_missing_entries_to_the_user
  end; alias run autostart_what # === run

  # ========================================================================= #
  # === orange
  # ========================================================================= #
  def orange(i)
    Colours.orange(i)
  end

  # ========================================================================= #
  # === Autostart.autostart_kde
  # ========================================================================= #
  def self.autostart_kde
    self.new(:kde)
  end

  # ========================================================================= #
  # === Autostart.run
  # ========================================================================= #
  def self.run(i = ARGV)
    self.new(i)
  end

end

if __FILE__ == $PROGRAM_NAME
  Autostart.new(ARGV.first)
end # autostart xfce